using Microsoft.EntityFrameworkCore;
using SampleEFCore8CodeMag;

namespace TestProject
{
    [TestClass]
    public class UnitTests
    {
        private readonly MyDbContext _context;

        public UnitTests()
        {
            var options = new DbContextOptionsBuilder<MyDbContext>()
                .UseSqlite("Filename=SampleEFCore8CodeMag")
                .Options;
            var options2=new DbContextOptionsBuilder<MyDbContext>()
                .UseSqlServer("Server=(localdb)\\mssqllocaldb;Database=SampleEFCore8CodeMag;Trusted_Connection=True;MultipleActiveResultSets=true")
                .Options;

            _context = new MyDbContext(options);
            _context.Database.EnsureDeleted();
            _context.Database.EnsureCreated();
        }
        [TestMethod]
        public void CanCreateWithCtorAndRetrieveCustomer()
        {
            var customer = new Customer
            {
                FirstPurchase = new DateOnly(2021, 1, 1),
                Name = new PersonName("John", "Doe")
            };
            _context.Customers.Add(customer);
            _context.SaveChanges();
            _context.ChangeTracker.Clear();
            var storedCustomer = _context.Customers.First();
            Assert.AreEqual("John", storedCustomer.Name.FirstName);
        }

        [TestMethod]
        public void CanCreateWithExpressionAndRetrieveCustomer()
        {
#if false //only possible when you have a default public ctor available
          //change compiler condition to true and notice that the expression builder won't compile
            var customer = new Customer
            {
                FirstPurchase = new DateOnly(2021, 1, 1),
                Name = new PersonName { FirstName = "John", LastName = "Doe" }
            };
            _context.Customers.Add(customer);
            _context.SaveChanges();
            _context.ChangeTracker.Clear();
            var storedCustomer = _context.Customers.First();
            Assert.AreEqual("John", storedCustomer.Name.FirstName);
#endif
        }

        [TestMethod]
        public void CanCreateCustomerWithNullName()
        {
            var customer = new Customer
            {
                FirstPurchase = new DateOnly(2021, 1, 1)
            };
            _context.Customers.Add(customer);
            _context.SaveChanges();
            _context.ChangeTracker.Clear();
            var storedCustomer = _context.Customers.First();
            Assert.IsNull(storedCustomer.Name);
        }
       
        
        //this will fail with shiplable.name as owned entity, 
        //pass when its a complexproperty
        [TestMethod]
        public void CanReuseCustomerNameWithTwoNewLabels()
        {
            var customer = new Customer
            {
                //define personname with expression builder or ctor
             #if false
                Name=new PersonName{FirstName="John", LastName="Doe"},
             #endif
                #if true
                Name = new PersonName("John", "Doe"),
                #endif
                FirstPurchase = new DateOnly(2021, 1, 1)
            };
            _context.Customers.Add(customer);
            _context.SaveChanges();
            _context.ChangeTracker.Clear();
            var storedCustomer = _context.Customers.First();
            var label = new ShipLabel
            {
                Name = storedCustomer.Name,
            };
            var label2 = new ShipLabel
            {
                Name = storedCustomer.Name,
            };

            _context.AddRange(label, label2);
            var debugview=_context.ChangeTracker.DebugView.LongView;
            _context.SaveChanges();
            _context.ChangeTracker.Clear();
            var storedLabels = _context.Labels.ToList();
            Assert.AreEqual("Doe", storedLabels[0].Name.LastName);
            Assert.AreEqual("Doe", storedLabels[1].Name.LastName);

        }
    }
}